def deer_iteration(
    invlin: Callable[[List[jnp.ndarray], jnp.ndarray, Any], jnp.ndarray],
    func: Callable[[List[jnp.ndarray], Any, Any], jnp.ndarray],
    shifter_func: Callable[[jnp.ndarray, Any], List[jnp.ndarray]],
    p_num: int,
    params: Any,  # gradable
    xinput: Any,  # gradable
    invlin_params: Any,  # gradable
    shifter_func_params: Any,  # gradable
    yinit_guess: jnp.ndarray,
    max_iter: int = 100,
    ) -> Tuple[jnp.ndarray, List[jnp.ndarray], Callable]:
  # obtain the functions to compute the jacobians and the function
  jacfunc = jax.vmap(jax.jacfwd(func, argnums=0), in_axes=(0, 0, None))
  func2 = jax.vmap(func, in_axes=(0, 0, None))

  dtype = yinit_guess.dtype
  # set the tolerance
  tol = 1e-7 if dtype == jnp.float64 else 1e-4

  def iter_func(iter_inp):
    err, yt, iiter = iter_inp
    # yt: (nsamples, ndims)
    ytparams = shifter_func(yt, shifter_func_params)
    # [p_num] + (nsamples, ndims, ndims)
    gts = [-gt for gt in jacfunc(ytparams, xinput, params)]  # FUNCEVAL
    # rhs: (nsamples, ndims)
    rhs = func2(ytparams, xinput, params)  # FUNCEVAL
    rhs += sum([jnp.einsum("...ij,...j->...i", gt, ytp)
                for gt, ytp in zip(gts, ytparams)])  # GTMULT
    # yt_next: (nsamples, ndims)
    yt_next = invlin(gts, rhs, invlin_params)  # INVLIN
    err = jnp.max(jnp.abs(yt_next - yt))  # checking convergence
    return err, yt_next, iiter + 1

  def cond_func(iter_inp) -> bool:
    err, _, iiter = iter_inp
    return jnp.logical_and(err > tol, iiter < max_iter)

  err = jnp.array(1e10, dtype=dtype)  # very high initial error
  gt = jnp.zeros((yinit_guess.shape[0], yinit_guess.shape[-1],
                  yinit_guess.shape[-1]), dtype=dtype)
  gts = [gt] * p_num
  iiter = jnp.array(0, dtype=jnp.int32)
  err, yt, iiter = jax.lax.while_loop(
    cond_func, iter_func, (err, yinit_guess, iiter))
  return yt
